@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Owning Modules:
       bld.c
@@ -120,6 +121,17 @@ typedef struct _ffebld_pool_stack_ *ffebldPoolstack_;
 #include "symbol.h"
 #include "target.h"
 
+#define FFEBLD_whereconstPROGUNIT_ 1
+#define FFEBLD_whereconstFILE_ 2
+
+#if FFECOM_targetCURRENT == FFECOM_targetFFE
+#define FFEBLD_whereconstCURRENT_ FFEBLD_whereconstPROGUNIT_
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
+#define FFEBLD_whereconstCURRENT_ FFEBLD_whereconstFILE_
+#else
+#error
+#endif
+
 /* Structure definitions. */
 
 #define FFEBLD_constINTEGERDEFAULT FFEBLD_constINTEGER1
@@ -721,7 +733,13 @@ ffetargetCharacterSize ffebld_size_max (ffebld b);
 #define ffebld_arrter(b) ((b)->u.arrter.array)
 #define ffebld_arrter_set_size(b,s) ((b)->u.arrter.size = (s))
 #define ffebld_arrter_size(b) ((b)->u.arrter.size)
-#define ffebld_constant_character_pool() ffe_pool_program_unit()
+#if FFEBLD_whereconstCURRENT_ == FFEBLD_whereconstPROGUNIT_
+#define ffebld_constant_pool() ffe_pool_program_unit()
+#elif FFEBLD_whereconstCURRENT_ == FFEBLD_whereconstFILE_
+#define ffebld_constant_pool() ffe_pool_file()
+#else
+#error
+#endif
 #define ffebld_constant_character1(c) ((c)->u.character1)
 #define ffebld_constant_character2(c) ((c)->u.character2)
 #define ffebld_constant_character3(c) ((c)->u.character3)
