@@ -164,7 +164,7 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 typedef enum
   {
-#define DEFGFRT(CODE,NAME,TYPE,VOLATILE,COMPLEX) CODE,
+#define DEFGFRT(CODE,NAME,TYPE,ARGS,VOLATILE,COMPLEX) CODE,
 #include "com-rt.def"
 #undef DEFGFRT
     FFECOM_gfrt
@@ -201,8 +201,9 @@ typedef struct _ffecom_symbol_ ffecomSymbol;
 struct _ffecom_symbol_
   {
     tree decl_tree;
-    tree length_tree;		/* For CHARACTER dummies and ASSIGN'ed vars. */
+    tree length_tree;		/* For CHARACTER dummies. */
     tree vardesc_tree;		/* For NAMELIST. */
+    tree assign_tree;		/* For ASSIGN'ed vars. */
     bool addr;			/* Is address of item instead of item. */
   };
 #endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
@@ -237,6 +238,7 @@ extern tree ffecom_f2c_real_type_node;
 extern tree ffecom_f2c_doublereal_type_node;
 extern tree ffecom_f2c_complex_type_node;
 extern tree ffecom_f2c_doublecomplex_type_node;
+extern tree ffecom_f2c_longint_type_node;
 extern tree ffecom_f2c_logical_type_node;
 extern tree ffecom_f2c_flag_type_node;
 extern tree ffecom_f2c_ftnlen_type_node;
