@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
 
@@ -39,6 +40,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "malloc.h"
 #include "st.h"
 #include "storag.h"
+#include "top.h"
 
 /* Externals defined here. */
 
@@ -52,9 +54,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    doesn't put the one initialized variable in a common area that has
    a large uninitialized array in it, while BIG-1.f does.  The size of
    the array is this many elements, as long as they all are INTEGER
-   type.  */
+   type.  Note that, as of 0.5.18, sparse cases are better handled,
+   so BIG-2.f now is used; it provides nonzero initial
+   values for all elements of the same array BIG-0 has.  */
 #ifndef FFEDATA_sizeTOO_BIG_INIT_
-#define FFEDATA_sizeTOO_BIG_INIT_ 25*1024
+#define FFEDATA_sizeTOO_BIG_INIT_ 75*1024
 #endif
 
 /* Internal typedefs. */
@@ -91,6 +95,7 @@ struct _ffedata_impdo_
 
 static ffedataImpdo_ ffedata_stack_ = NULL;
 static ffebld ffedata_list_ = NULL;
+static bool ffedata_reinit_;	/* value_ should report REINIT error. */
 static bool ffedata_reported_error_;	/* Error has been reported. */
 static ffesymbol ffedata_symbol_ = NULL;	/* Symbol being initialized. */
 static ffeinfoBasictype ffedata_basictype_;	/* Info on symbol. */
@@ -150,6 +155,7 @@ ffedata_begin (ffebld list)
   ffedata_list_ = list;
   ffedata_symbol_ = NULL;
   ffedata_reported_error_ = FALSE;
+  ffedata_reinit_ = FALSE;
   ffedata_advance_ ();
 }
 
@@ -211,17 +217,20 @@ ffedata_gather (ffestorag st)
   /* Prepare info on the storage area we're putting init info into. */
 
   ffetarget_aggregate_info (&ffedata_storage_bt_, &ffedata_storage_kt_,
-			    &ffedata_storage_units_, ffestorag_basictype (st), ffestorag_kindtype (st));
+			    &ffedata_storage_units_, ffestorag_basictype (st),
+			    ffestorag_kindtype (st));
   ffedata_storage_size_ = ffestorag_size (st) / ffedata_storage_units_;
   assert (ffestorag_size (st) % ffedata_storage_units_ == 0);
 
   /* If a CBLOCK, gather all the init info for its explicit members. */
 
-  s = ffestorag_symbol (st);
-  if (s != NULL)
+  if ((ffestorag_type (st) == FFESTORAG_typeCBLOCK)
+      && (ffestorag_symbol (st) != NULL))
     {
+      s = ffestorag_symbol (st);
       for (b = ffesymbol_commonlist (s); b != NULL; b = ffebld_trail (b))
-	ffedata_gather_ (st, ffesymbol_storage (ffebld_symter (ffebld_head (b))));
+	ffedata_gather_ (st,
+			 ffesymbol_storage (ffebld_symter (ffebld_head (b))));
     }
 
   /* For CBLOCK or LOCAL, gather all the init info for equivalenced members. */
@@ -245,6 +254,24 @@ ffedata_value (ffetargetIntegerDefault rpt, ffebld value, ffelexToken token)
 {
   ffetargetIntegerDefault i;
 
+  /* Maybe ignore zero values, to speed up compiling, even though we lose
+     checking for multiple initializations for now.  */
+
+  if (!ffe_is_zeros ()
+      && (value != NULL)
+      && (ffebld_op (value) == FFEBLD_opCONTER)
+      && ffebld_constant_is_zero (ffebld_conter (value)))
+    value = NULL;
+  else if ((value != NULL)
+	   && (ffebld_op (value) == FFEBLD_opANY))
+    value = NULL;
+  else
+    {
+      /* Must be a constant. */
+      assert (value != NULL);
+      assert (ffebld_op (value) == FFEBLD_opCONTER);
+    }
+
   /* Later we can optimize certain cases by seeing that the target array can
      take some number of values, and provide this number to _value_. */
 
@@ -254,8 +281,24 @@ ffedata_value (ffetargetIntegerDefault rpt, ffebld value, ffelexToken token)
     ffedata_convert_cache_use_ = 0;	/* Maybe use the cache. */
 
   for (i = 0; i < rpt; ++i)
-    if (!ffedata_value_ (value, token))
-      return FALSE;
+    {
+      if ((ffedata_symbol_ != NULL)
+	  && !ffesymbol_is_init (ffedata_symbol_))
+	{
+	  ffesymbol_signal_change (ffedata_symbol_);
+	  ffesymbol_update_init (ffedata_symbol_);
+	  if (ffe_is_90 ())
+	    ffesymbol_update_save (ffedata_symbol_);
+#if FFEGLOBAL_ENABLED
+	  if (ffesymbol_common (ffedata_symbol_) != NULL)
+	    ffeglobal_init_common (ffesymbol_common (ffedata_symbol_),
+				   token);
+#endif
+	  ffesymbol_signal_unreported (ffedata_symbol_);
+	}
+      if (!ffedata_value_ (value, token))
+	return FALSE;
+    }
 
   return TRUE;
 }
@@ -335,9 +378,6 @@ tail_recurse:			/* :::::::::::::::::::: */
     {
     case FFEBLD_opSYMTER:	/* Simple reference to scalar or array. */
       ffedata_symbol_ = ffebld_symter (next);
-#if 0				/* Fortran 90 only, someday.... */
-      ffesymbol_update_save (ffedata_symbol_);
-#endif
       ffedata_storage_ = (ffesymbol_storage (ffedata_symbol_) == NULL) ? NULL
 	: ffestorag_parent (ffesymbol_storage (ffedata_symbol_));
       if (ffedata_storage_ != NULL)
@@ -356,12 +396,17 @@ tail_recurse:			/* :::::::::::::::::::: */
 	  || ((ffedata_storage_ != NULL)
 	      && (ffestorag_init (ffedata_storage_) != NULL)))
 	{
+#if 0
 	  ffebad_start (FFEBAD_DATA_REINIT);
 	  ffest_ffebad_here_current_stmt (0);
 	  ffebad_string (ffesymbol_text (ffedata_symbol_));
 	  ffebad_finish ();
 	  ffedata_reported_error_ = TRUE;
 	  return FALSE;
+#else
+	  ffedata_reinit_ = TRUE;
+	  return TRUE;
+#endif
 	}
       ffedata_basictype_ = ffesymbol_basictype (ffedata_symbol_);
       ffedata_kindtype_ = ffesymbol_kindtype (ffedata_symbol_);
@@ -393,9 +438,6 @@ tail_recurse:			/* :::::::::::::::::::: */
 
     case FFEBLD_opARRAYREF:	/* Reference to element of array. */
       ffedata_symbol_ = ffebld_symter (ffebld_left (next));
-#if 0				/* Fortran 90 only, someday.... */
-      ffesymbol_update_save (ffedata_symbol_);
-#endif
       ffedata_storage_ = (ffesymbol_storage (ffedata_symbol_) == NULL) ? NULL
 	: ffestorag_parent (ffesymbol_storage (ffedata_symbol_));
       if (ffedata_storage_ != NULL)
@@ -413,12 +455,17 @@ tail_recurse:			/* :::::::::::::::::::: */
 	  || ((ffedata_storage_ != NULL)
 	      && (ffestorag_init (ffedata_storage_) != NULL)))
 	{
+#if 0
 	  ffebad_start (FFEBAD_DATA_REINIT);
 	  ffest_ffebad_here_current_stmt (0);
 	  ffebad_string (ffesymbol_text (ffedata_symbol_));
 	  ffebad_finish ();
 	  ffedata_reported_error_ = TRUE;
 	  return FALSE;
+#else
+	  ffedata_reinit_ = TRUE;
+	  return TRUE;
+#endif
 	}
       ffedata_basictype_ = ffesymbol_basictype (ffedata_symbol_);
       ffedata_kindtype_ = ffesymbol_kindtype (ffedata_symbol_);
@@ -459,9 +506,6 @@ tail_recurse:			/* :::::::::::::::::::: */
 
 	ffedata_symbol_ = ffebld_symter (ffebld_left (arrayref
 					      ? ffebld_left (next) : next));
-#if 0				/* Fortran 90 only, someday.... */
-	ffesymbol_update_save (ffedata_symbol_);
-#endif
 	ffedata_storage_ = (ffesymbol_storage (ffedata_symbol_) == NULL) ? NULL
 	  : ffestorag_parent (ffesymbol_storage (ffedata_symbol_));
 	if (ffedata_storage_ != NULL)
@@ -479,12 +523,17 @@ tail_recurse:			/* :::::::::::::::::::: */
 	    || ((ffedata_storage_ != NULL)
 		&& (ffestorag_init (ffedata_storage_) != NULL)))
 	  {
+#if 0
 	    ffebad_start (FFEBAD_DATA_REINIT);
 	    ffest_ffebad_here_current_stmt (0);
 	    ffebad_string (ffesymbol_text (ffedata_symbol_));
 	    ffebad_finish ();
 	    ffedata_reported_error_ = TRUE;
 	    return FALSE;
+#else
+	    ffedata_reinit_ = TRUE;
+	    return TRUE;
+#endif
 	  }
 	ffedata_basictype_ = ffesymbol_basictype (ffedata_symbol_);
 	ffedata_kindtype_ = ffesymbol_kindtype (ffedata_symbol_);
@@ -1391,9 +1440,8 @@ ffedata_value_ (ffebld value, ffelexToken token)
   /* If the value is an error marker, remember we've seen one and do nothing
      else. */
 
-  assert (value != NULL);
-
-  if (ffebld_op (value) == FFEBLD_opANY)
+  if ((value != NULL)
+      && (ffebld_op (value) == FFEBLD_opANY))
     {
       ffedata_reported_error_ = TRUE;
       return FALSE;
@@ -1411,32 +1459,48 @@ ffedata_value_ (ffebld value, ffelexToken token)
       return FALSE;
     }
 
+  /* If ffedata_advance_ wanted to register a complaint, do it now
+     that we have the token to point at instead of just the start
+     of the whole statement.  */
+
+  if (ffedata_reinit_)
+    {
+      ffebad_start (FFEBAD_DATA_REINIT);
+      ffebad_here (0, ffelex_token_where_line (token),
+		   ffelex_token_where_column (token));
+      ffebad_string (ffesymbol_text (ffedata_symbol_));
+      ffebad_finish ();
+      ffedata_reported_error_ = TRUE;
+      return FALSE;
+    }
+
 #if FFEGLOBAL_ENABLED
   if (ffesymbol_common (ffedata_symbol_) != NULL)
     ffeglobal_init_common (ffesymbol_common (ffedata_symbol_), token);
 #endif
 
-  /* Must be a constant. */
-
-  assert (ffebld_op (value) == FFEBLD_opCONTER);
-
   /* Convert value to desired type. */
 
-  if (ffedata_convert_cache_use_ == -1)
-    value = ffeexpr_convert (value, token, NULL, ffedata_basictype_,
-			     ffedata_kindtype_, 0,
-			  (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
-		      ? ffedata_charexpected_ : FFETARGET_charactersizeNONE,
-			     FFEEXPR_contextDATA);
-  else				/* Use the cache. */
-    value = ffedata_convert_ (value, token, NULL, ffedata_basictype_,
-			      ffedata_kindtype_, 0,
-			  (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
-		     ? ffedata_charexpected_ : FFETARGET_charactersizeNONE);
+  if (value != NULL)
+    {
+      if (ffedata_convert_cache_use_ == -1)
+	value = ffeexpr_convert
+	  (value, token, NULL, ffedata_basictype_,
+	   ffedata_kindtype_, 0,
+	   (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
+	   ? ffedata_charexpected_ : FFETARGET_charactersizeNONE,
+	   FFEEXPR_contextDATA);
+      else				/* Use the cache. */
+	value = ffedata_convert_
+	  (value, token, NULL, ffedata_basictype_,
+	   ffedata_kindtype_, 0,
+	   (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
+	   ? ffedata_charexpected_ : FFETARGET_charactersizeNONE);
+    }
 
   /* If we couldn't, bug out. */
 
-  if ((value == NULL) || (ffebld_op (value) == FFEBLD_opANY))
+  if ((value != NULL) && (ffebld_op (value) == FFEBLD_opANY))
     {
       ffedata_reported_error_ = TRUE;
       return FALSE;
@@ -1503,83 +1567,97 @@ ffedata_value_ (ffebld value, ffelexToken token)
 
       /* Does an accretion array exist?	 If not, create it. */
 
-      if (ffestorag_accretion (ffedata_storage_) == NULL)
+      if (value != NULL)
 	{
-#if FFEDATA_sizeTOO_BIG_INIT_ != 0
-	  if (ffedata_storage_size_ >= FFEDATA_sizeTOO_BIG_INIT_)
+	  if (ffestorag_accretion (ffedata_storage_) == NULL)
 	    {
-	      char bignum[40];
+#if FFEDATA_sizeTOO_BIG_INIT_ != 0
+	      if (ffedata_storage_size_ >= FFEDATA_sizeTOO_BIG_INIT_)
+		{
+		  char bignum[40];
 
-	      sprintf (&bignum[0], "%ld", (long) ffedata_storage_size_);
-	      ffebad_start (FFEBAD_TOO_BIG_INIT);
+		  sprintf (&bignum[0], "%ld", (long) ffedata_storage_size_);
+		  ffebad_start (FFEBAD_TOO_BIG_INIT);
+		  ffebad_here (0, ffelex_token_where_line (token),
+			       ffelex_token_where_column (token));
+		  ffebad_string (ffesymbol_text (ffedata_symbol_));
+		  ffebad_string (bignum);
+		  ffebad_finish ();
+		}
+#endif
+	      array = ffebld_constantarray_new
+		(ffedata_storage_bt_, ffedata_storage_kt_,
+		 ffedata_storage_size_);
+	      accter = ffebld_new_accter (array,
+					  ffebit_new (ffe_pool_program_unit (),
+						      ffedata_storage_size_));
+	      ffebld_set_info (accter, ffeinfo_new
+			       (ffedata_storage_bt_,
+				ffedata_storage_kt_,
+				1,
+				FFEINFO_kindENTITY,
+				FFEINFO_whereCONSTANT,
+				(ffedata_basictype_
+				 == FFEINFO_basictypeCHARACTER)
+				? 1 : FFETARGET_charactersizeNONE));
+	      ffestorag_set_accretion (ffedata_storage_, accter);
+	      ffestorag_set_accretes (ffedata_storage_, ffedata_storage_size_);
+	    }
+	  else
+	    {
+	      accter = ffestorag_accretion (ffedata_storage_);
+	      assert (ffedata_storage_size_ == ffebld_accter_size (accter));
+	      array = ffebld_accter (accter);
+	    }
+
+	  /* Put value in accretion array at desired offset. */
+
+	  fn = ffetarget_aggregate_ptr_memcpy
+	    (ffedata_storage_bt_, ffedata_storage_kt_,
+	     ffedata_basictype_, ffedata_kindtype_);
+	  ffebld_constantarray_prepare
+	    (&ptr1, &ptr2, &size, array, ffedata_storage_bt_,
+	     ffedata_storage_kt_, offset,
+	     ffebld_constant_ptr_to_union (ffebld_conter (value)),
+	     ffedata_basictype_, ffedata_kindtype_);
+	  (*fn) (ptr1, ptr2, size);	/* Does the appropriate memcpy-like
+					   operation. */
+	  ffebit_count (ffebld_accter_bits (accter),
+			offset, FALSE, units_expected,
+			&actual);	/* How many FALSE? */
+	  if (actual != units_expected)
+	    {
+	      ffebad_start (FFEBAD_DATA_MULTIPLE);
 	      ffebad_here (0, ffelex_token_where_line (token),
 			   ffelex_token_where_column (token));
 	      ffebad_string (ffesymbol_text (ffedata_symbol_));
-	      ffebad_string (bignum);
 	      ffebad_finish ();
 	    }
-#endif
-	  array = ffebld_constantarray_new (ffedata_storage_bt_,
-				ffedata_storage_kt_, ffedata_storage_size_);
-	  accter = ffebld_new_accter (array, ffebit_new (ffe_pool_program_unit (),
-						    ffedata_storage_size_));
-	  ffebld_set_info (accter, ffeinfo_new
-			   (ffedata_storage_bt_,
-			    ffedata_storage_kt_,
-			    1,
-			    FFEINFO_kindENTITY,
-			    FFEINFO_whereCONSTANT,
-			  (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
-			    ? 1 : FFETARGET_charactersizeNONE));
-	  ffestorag_set_accretion (ffedata_storage_, accter);
-	  ffestorag_set_accretes (ffedata_storage_, ffedata_storage_size_);
-	}
-      else
-	{
-	  accter = ffestorag_accretion (ffedata_storage_);
-	  assert (ffedata_storage_size_ == ffebld_accter_size (accter));
-	  array = ffebld_accter (accter);
-	}
-
-      /* Put value in accretion array at desired offset. */
-
-      fn = ffetarget_aggregate_ptr_memcpy (ffedata_storage_bt_,
-		ffedata_storage_kt_, ffedata_basictype_, ffedata_kindtype_);
-      ffebld_constantarray_prepare (&ptr1, &ptr2, &size, array, ffedata_storage_bt_,
-				    ffedata_storage_kt_, offset,
-		       ffebld_constant_ptr_to_union (ffebld_conter (value)),
-				    ffedata_basictype_, ffedata_kindtype_);
-      (*fn) (ptr1, ptr2, size);	/* Does the appropriate memcpy-like
-				   operation. */
-      ffebit_count (ffebld_accter_bits (accter),
-		    offset, FALSE, units_expected, &actual);	/* How many FALSE? */
-      if (actual != units_expected)
-	{
-	  ffebad_start (FFEBAD_DATA_MULTIPLE);
-	  ffebad_here (0, ffelex_token_where_line (token),
-		       ffelex_token_where_column (token));
-	  ffebad_string (ffesymbol_text (ffedata_symbol_));
-	  ffebad_finish ();
-	}
-      ffestorag_set_accretes (ffedata_storage_,
-			      ffestorag_accretes (ffedata_storage_)
-			      - actual);	/* Decrement # of values
+	  ffestorag_set_accretes (ffedata_storage_,
+				  ffestorag_accretes (ffedata_storage_)
+				  - actual);	/* Decrement # of values
 						   actually accreted. */
-      ffebit_set (ffebld_accter_bits (accter), offset, 1, units_expected);
+	  ffebit_set (ffebld_accter_bits (accter), offset,
+		      1, units_expected);
 
-      /* If done accreting for this storage area, establish as initialized. */
+	  /* If done accreting for this storage area, establish as
+	     initialized. */
 
-      if (ffestorag_accretes (ffedata_storage_) == 0)
-	{
-	  ffestorag_set_init (ffedata_storage_, accter);
-	  ffestorag_set_accretion (ffedata_storage_, NULL);
-	  ffebit_kill (ffebld_accter_bits (ffestorag_init (ffedata_storage_)));
-	  ffebld_set_op (ffestorag_init (ffedata_storage_), FFEBLD_opARRTER);
-	  ffebld_set_arrter (ffestorag_init (ffedata_storage_),
-			 ffebld_accter (ffestorag_init (ffedata_storage_)));
-	  ffebld_arrter_set_size (ffestorag_init (ffedata_storage_),
-				  ffedata_storage_size_);
-	  ffecom_notify_init_storage (ffedata_storage_);
+	  if (ffestorag_accretes (ffedata_storage_) == 0)
+	    {
+	      ffestorag_set_init (ffedata_storage_, accter);
+	      ffestorag_set_accretion (ffedata_storage_, NULL);
+	      ffebit_kill (ffebld_accter_bits
+			   (ffestorag_init (ffedata_storage_)));
+	      ffebld_set_op (ffestorag_init (ffedata_storage_),
+			     FFEBLD_opARRTER);
+	      ffebld_set_arrter
+		(ffestorag_init (ffedata_storage_),
+		 ffebld_accter (ffestorag_init (ffedata_storage_)));
+	      ffebld_arrter_set_size (ffestorag_init (ffedata_storage_),
+				      ffedata_storage_size_);
+	      ffecom_notify_init_storage (ffedata_storage_);
+	    }
 	}
 
       /* If still accreting, adjust specs accordingly and return. */
@@ -1596,13 +1674,15 @@ ffedata_value_ (ffebld value, ffelexToken token)
   /* Figure out where the value goes -- in an accretion array or directly
      into the final initial-value slot for the symbol. */
 
-  if ((ffedata_number_ != 0) || (ffedata_arraysize_ > 1)
-   || (ffedata_charnumber_ != 0) || (ffedata_size_ > ffedata_charexpected_))
+  if ((ffedata_number_ != 0)
+      || (ffedata_arraysize_ > 1)
+      || (ffedata_charnumber_ != 0)
+      || (ffedata_size_ > ffedata_charexpected_))
     {				/* Accrete this value. */
       ffetargetOffset offset;
       ffebitCount actual;
       ffebldConstantArray array;
-      ffebld accter;
+      ffebld accter = NULL;
 
       /* Calculate offset. */
 
@@ -1622,65 +1702,74 @@ ffedata_value_ (ffebld value, ffelexToken token)
 
       /* Does an accretion array exist?	 If not, create it. */
 
-      if (ffesymbol_accretion (ffedata_symbol_) == NULL)
+      if (value != NULL)
 	{
-#if FFEDATA_sizeTOO_BIG_INIT_ != 0
-	  if (ffedata_symbolsize_ >= FFEDATA_sizeTOO_BIG_INIT_ )
+	  if (ffesymbol_accretion (ffedata_symbol_) == NULL)
 	    {
-	      char bignum[40];
+#if FFEDATA_sizeTOO_BIG_INIT_ != 0
+	      if (ffedata_symbolsize_ >= FFEDATA_sizeTOO_BIG_INIT_ )
+		{
+		  char bignum[40];
 
-	      sprintf (&bignum[0], "%ld", (long) ffedata_symbolsize_);
-	      ffebad_start (FFEBAD_TOO_BIG_INIT);
+		  sprintf (&bignum[0], "%ld", (long) ffedata_symbolsize_);
+		  ffebad_start (FFEBAD_TOO_BIG_INIT);
+		  ffebad_here (0, ffelex_token_where_line (token),
+			       ffelex_token_where_column (token));
+		  ffebad_string (ffesymbol_text (ffedata_symbol_));
+		  ffebad_string (bignum);
+		  ffebad_finish ();
+		}
+#endif
+	      array = ffebld_constantarray_new
+		(ffedata_basictype_, ffedata_kindtype_,
+		 ffedata_symbolsize_);
+	      accter = ffebld_new_accter (array,
+					  ffebit_new (ffe_pool_program_unit (),
+						      ffedata_symbolsize_));
+	      ffebld_set_info (accter, ffeinfo_new
+			       (ffedata_basictype_,
+				ffedata_kindtype_,
+				1,
+				FFEINFO_kindENTITY,
+				FFEINFO_whereCONSTANT,
+				(ffedata_basictype_
+				 == FFEINFO_basictypeCHARACTER)
+				? 1 : FFETARGET_charactersizeNONE));
+	      ffesymbol_set_accretion (ffedata_symbol_, accter);
+	      ffesymbol_set_accretes (ffedata_symbol_, ffedata_symbolsize_);
+	    }
+	  else
+	    {
+	      accter = ffesymbol_accretion (ffedata_symbol_);
+	      assert (ffedata_symbolsize_
+		      == ffebld_accter_size (accter));
+	      array = ffebld_accter (accter);
+	    }
+
+	  /* Put value in accretion array at desired offset. */
+
+	  ffebld_constantarray_put
+	    (array, ffedata_basictype_, ffedata_kindtype_,
+	     offset, ffebld_constant_union (ffebld_conter (value)));
+	  ffebit_count (ffebld_accter_bits (accter), offset, FALSE,
+			ffedata_charexpected_,
+			&actual);	/* How many FALSE? */
+	  if (actual != (unsigned long int) ffedata_charexpected_)
+	    {
+	      ffebad_start (FFEBAD_DATA_MULTIPLE);
 	      ffebad_here (0, ffelex_token_where_line (token),
 			   ffelex_token_where_column (token));
 	      ffebad_string (ffesymbol_text (ffedata_symbol_));
-	      ffebad_string (bignum);
 	      ffebad_finish ();
 	    }
-#endif
-	  array = ffebld_constantarray_new (ffedata_basictype_, ffedata_kindtype_,
-					    ffedata_symbolsize_);
-	  accter = ffebld_new_accter (array, ffebit_new (ffe_pool_program_unit (),
-						      ffedata_symbolsize_));
-	  ffebld_set_info (accter, ffeinfo_new
-			   (ffedata_basictype_,
-			    ffedata_kindtype_,
-			    1,
-			    FFEINFO_kindENTITY,
-			    FFEINFO_whereCONSTANT,
-			  (ffedata_basictype_ == FFEINFO_basictypeCHARACTER)
-			    ? 1 : FFETARGET_charactersizeNONE));
-	  ffesymbol_set_accretion (ffedata_symbol_, accter);
-	  ffesymbol_set_accretes (ffedata_symbol_, ffedata_symbolsize_);
-	}
-      else
-	{
-	  accter = ffesymbol_accretion (ffedata_symbol_);
-	  assert (ffedata_symbolsize_
-		  == ffebld_accter_size (accter));
-	  array = ffebld_accter (accter);
-	}
-
-      /* Put value in accretion array at desired offset. */
-
-      ffebld_constantarray_put (array, ffedata_basictype_, ffedata_kindtype_,
-		     offset, ffebld_constant_union (ffebld_conter (value)));
-      ffebit_count (ffebld_accter_bits (accter),
-		    offset, FALSE, ffedata_charexpected_, &actual);	/* How many FALSE? */
-      if (actual != ffedata_charexpected_)
-	{
-	  ffebad_start (FFEBAD_DATA_MULTIPLE);
-	  ffebad_here (0, ffelex_token_where_line (token),
-		       ffelex_token_where_column (token));
-	  ffebad_string (ffesymbol_text (ffedata_symbol_));
-	  ffebad_finish ();
-	}
-      ffesymbol_set_accretes (ffedata_symbol_, ffesymbol_accretes (ffedata_symbol_)
-			      - actual);	/* Decrement # of values
+	  ffesymbol_set_accretes (ffedata_symbol_,
+				  ffesymbol_accretes (ffedata_symbol_)
+				  - actual);	/* Decrement # of values
 						   actually accreted. */
-      ffebit_set (ffebld_accter_bits (accter), offset,
-		  1, ffedata_charexpected_);
-      ffesymbol_signal_unreported (ffedata_symbol_);
+	  ffebit_set (ffebld_accter_bits (accter), offset,
+		      1, ffedata_charexpected_);
+	  ffesymbol_signal_unreported (ffedata_symbol_);
+	}
 
       /* If still accreting, adjust specs accordingly and return. */
 
@@ -1692,7 +1781,8 @@ ffedata_value_ (ffebld value, ffelexToken token)
 
       /* Else, if done accreting for this symbol, establish as initialized. */
 
-      if (ffesymbol_accretes (ffedata_symbol_) == 0)
+      if ((value != NULL)
+	  && (ffesymbol_accretes (ffedata_symbol_) == 0))
 	{
 	  ffesymbol_set_init (ffedata_symbol_, accter);
 	  ffesymbol_set_accretion (ffedata_symbol_, NULL);
@@ -1704,11 +1794,10 @@ ffedata_value_ (ffebld value, ffelexToken token)
 				  ffedata_symbolsize_);
 	  ffecom_notify_init_symbol (ffedata_symbol_);
 	}
-
     }
-  else
-    /* Simple, direct, one-shot assignment. */
+  else if (value != NULL)
     {
+      /* Simple, direct, one-shot assignment. */
       ffesymbol_set_init (ffedata_symbol_, value);
       ffecom_notify_init_symbol (ffedata_symbol_);
     }
