@@ -95,4 +95,8 @@ typedef enum
 #endif	/* !BUILT_WITH_270 */
 #endif  /* !defined (UNUSED) */
 
+#ifndef dmpout
+#define dmpout stderr
+#endif
+
 #endif
