@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -128,12 +129,12 @@ ffestt_caselist_dump (ffesttCaseList list)
   for (next = list->next; next != list; next = next->next)
     {
       if (next != list->next)
-	fputc (',', stdout);
+	fputc (',', dmpout);
       if (next->expr1 != NULL)
 	ffebld_dump (next->expr1);
       if (next->range)
 	{
-	  fputc (':', stdout);
+	  fputc (':', dmpout);
 	  if (next->expr2 != NULL)
 	    ffebld_dump (next->expr2);
 	}
@@ -433,10 +434,10 @@ ffestt_dimlist_dump (ffesttDimList list)
   for (next = list->next; next != list; next = next->next)
     {
       if (next != list->next)
-	fputc (',', stdout);
+	fputc (',', dmpout);
       if (next->lower != NULL)
 	ffebld_dump (next->lower);
-      fputc (':', stdout);
+      fputc (':', dmpout);
       if (next->upper != NULL)
 	ffebld_dump (next->upper);
     }
@@ -582,7 +583,7 @@ ffestt_exprlist_dump (ffesttExprList list)
   for (next = list->next; next != list; next = next->next)
     {
       if (next != list->next)
-	fputc (',', stdout);
+	fputc (',', dmpout);
       ffebld_dump (next->expr);
     }
 }
@@ -819,14 +820,14 @@ ffestt_implist_dump (ffesttImpList list)
   for (next = list->next; next != list; next = next->next)
     {
       if (next != list->next)
-	fputc (',', stdout);
+	fputc (',', dmpout);
       assert (ffelex_token_type (next->first) == FFELEX_typeNAME);
-      fputs (ffelex_token_text (next->first), stdout);
+      fputs (ffelex_token_text (next->first), dmpout);
       if (next->last != NULL)
 	{
-	  fputc ('-', stdout);
+	  fputc ('-', dmpout);
 	  assert (ffelex_token_type (next->last) == FFELEX_typeNAME);
-	  fputs (ffelex_token_text (next->last), stdout);
+	  fputs (ffelex_token_text (next->last), dmpout);
 	}
     }
 }
@@ -931,22 +932,22 @@ ffestt_tokenlist_dump (ffesttTokenList tl)
   for (ti = tl->first; ti != (ffesttTokenItem) &tl->first; ti = ti->next)
     {
       if (ti != tl->first)
-	fputc (',', stdout);
+	fputc (',', dmpout);
       switch (ffelex_token_type (ti->t))
 	{
 	case FFELEX_typeNUMBER:
 	case FFELEX_typeNAME:
 	case FFELEX_typeNAMES:
-	  fputs (ffelex_token_text (ti->t), stdout);
+	  fputs (ffelex_token_text (ti->t), dmpout);
 	  break;
 
 	case FFELEX_typeASTERISK:
-	  fputc ('*', stdout);
+	  fputc ('*', dmpout);
 	  break;
 
 	default:
 	  assert (FALSE);
-	  fputc ('?', stdout);
+	  fputc ('?', dmpout);
 	  break;
 	}
     }
