@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
 */
 
@@ -92,13 +93,10 @@ ffestu_sym_end_transition (ffesymbol s)
     {
     case FFESYMBOL_stateUNCERTAIN:
       if ((swh == FFEINFO_whereDUMMY)
-#if 0
-	  && (skd == FFEINFO_kindENTITY)
-	  && (sa & (FFESYMBOL_attrsADJUSTABLE | FFESYMBOL_attrsANYSIZE))
-#endif
 	  && (ffesymbol_numentries (s) == 0))
 	{			/* Not actually in any dummy list! */
 	  ffesymbol_error (s, ffesta_tokens[0]);
+	  return s;
 	}
       break;
 
@@ -109,13 +107,10 @@ ffestu_sym_end_transition (ffesymbol s)
 	ffestu_dummies_transition_ (ffecom_sym_end_transition,
 				    ffesymbol_dummyargs (s));
       else if ((swh == FFEINFO_whereDUMMY)
-#if 0
-	       && (skd == FFEINFO_kindENTITY)
-	       && (sa & (FFESYMBOL_attrsADJUSTABLE | FFESYMBOL_attrsANYSIZE))
-#endif
 	       && (ffesymbol_numentries (s) == 0))
 	{			/* Not actually in any dummy list! */
 	  ffesymbol_error (s, ffesta_tokens[0]);
+	  return s;
 	}
 
       ffestorag_end_layout (s);
@@ -131,6 +126,7 @@ ffestu_sym_end_transition (ffesymbol s)
   na = sa = ffesymbol_attrs (s);
 
   assert (!(sa & ~(FFESYMBOL_attrsACTUALARG
+		   | FFESYMBOL_attrsADJUSTABLE
 		   | FFESYMBOL_attrsANYLEN
 		   | FFESYMBOL_attrsARRAY
 		   | FFESYMBOL_attrsDUMMY
@@ -183,22 +179,35 @@ ffestu_sym_end_transition (ffesymbol s)
 		       | FFESYMBOL_attrsTYPE)));
 
       /* Honestly, this appears to be a guess.  I can't find anyplace in the
-         standard that makes clear whether this unreferenced dummy argument
-         is an ENTITY or a FUNCTION.  And yet, for the f2c interface, picking
-         one is critical for CHARACTER entities because it determines whether
-         to expect an additional argument specifying the length of an ENTITY
-         that is not expected (or needed) for a FUNCTION.  HOWEVER, F90 makes
-         this guess a correct one, and it does seem that the Section 18 Notes
-         in Appendix B of F77 make it clear the F77 standard at least
-         intended to make this guess correct as well, so this seems ok.  */
+	 standard that makes clear whether this unreferenced dummy argument
+	 is an ENTITY or a FUNCTION.  And yet, for the f2c interface, picking
+	 one is critical for CHARACTER entities because it determines whether
+	 to expect an additional argument specifying the length of an ENTITY
+	 that is not expected (or needed) for a FUNCTION.  HOWEVER, F90 makes
+	 this guess a correct one, and it does seem that the Section 18 Notes
+	 in Appendix B of F77 make it clear the F77 standard at least
+	 intended to make this guess correct as well, so this seems ok.  */
 
       nkd = FFEINFO_kindENTITY;
     }
   else if (sa & FFESYMBOL_attrsARRAY)
     {
       assert (!(sa & ~(FFESYMBOL_attrsARRAY
+		       | FFESYMBOL_attrsADJUSTABLE
 		       | FFESYMBOL_attrsTYPE)));
 
+      if (sa & FFESYMBOL_attrsADJUSTABLE)
+	{			/* Not actually in any dummy list! */
+	  if (ffe_is_pedantic ()
+	      && ffebad_start_msg ("Local adjustable symbol `%A' at %0",
+				   FFEBAD_severityPEDANTIC))
+	    {
+	      ffebad_string (ffesymbol_text (s));
+	      ffebad_here (0, ffesymbol_where_line (s),
+			   ffesymbol_where_column (s));
+	      ffebad_finish ();
+	    }
+	}
       nwh = FFEINFO_whereLOCAL;
     }
   else if (sa & FFESYMBOL_attrsSFARG)
@@ -214,12 +223,13 @@ ffestu_sym_end_transition (ffesymbol s)
 		      | FFESYMBOL_attrsDUMMY
 		      | FFESYMBOL_attrsEXTERNAL
 		      | FFESYMBOL_attrsSFARG)));	/* Handled above. */
-      assert (!(sa & ~(FFESYMBOL_attrsANYLEN
+      assert (!(sa & ~(FFESYMBOL_attrsTYPE
+		       | FFESYMBOL_attrsADJUSTABLE
+		       | FFESYMBOL_attrsANYLEN
 		       | FFESYMBOL_attrsARRAY
 		       | FFESYMBOL_attrsDUMMY
 		       | FFESYMBOL_attrsEXTERNAL
-		       | FFESYMBOL_attrsSFARG
-		       | FFESYMBOL_attrsTYPE)));
+		       | FFESYMBOL_attrsSFARG)));
 
       if (sa & FFESYMBOL_attrsANYLEN)
 	{			/* Can't touch this. */
@@ -586,10 +596,18 @@ ffestu_sym_exec_transition (ffesymbol s)
 		       | FFESYMBOL_attrsARRAY
 		       | FFESYMBOL_attrsTYPE)));
 
-      if (sa & FFESYMBOL_attrsADJUSTABLE)
-	ffestu_symter_exec_transition_ (ffesymbol_dims (s));
       nkd = FFEINFO_kindENTITY;
-      nwh = FFEINFO_whereDUMMY;
+
+      if (sa & FFESYMBOL_attrsADJUSTABLE)
+	  ffestu_symter_exec_transition_ (ffesymbol_dims (s));
+
+      if (sa & (FFESYMBOL_attrsANYLEN | FFESYMBOL_attrsANYSIZE))
+	nwh = FFEINFO_whereDUMMY;
+      else
+	{
+	  nwh = FFEINFO_whereNONE;	/* DUMMY, LOCAL. */
+	  ns = FFESYMBOL_stateUNCERTAIN;
+	}
     }
   else if (sa & FFESYMBOL_attrsARRAY)
     {
