@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.  */
 
 #include "proj.h"
 #include "symbol.h"
@@ -41,27 +42,29 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    easily be taken apart program unit by program unit, the latter is the
    UNIX/C model where the object file is essentially a monolith.  */
 
-#define FFESYMBOL_globalPROGUNIT_ 0
-#define FFESYMBOL_globalFILE_ 1
+#define FFESYMBOL_globalPROGUNIT_ 1
+#define FFESYMBOL_globalFILE_ 2
 
 /* Choose how to handle global symbols here.  */
 
 #if FFECOM_targetCURRENT == FFECOM_targetFFE
 #define FFESYMBOL_globalCURRENT_ FFESYMBOL_globalPROGUNIT_
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
+/* Would be good to understand why PROGUNIT in this case too.
+   (1995-08-22).  */
+#define FFESYMBOL_globalCURRENT_ FFESYMBOL_globalPROGUNIT_
 #else
-#if FFECOM_targetCURRENT == FFECOM_targetGCC
-#define FFESYMBOL_globalCURRENT_ FFESYMBOL_globalPROGRAMUNIT_
-#endif
+#error
 #endif
 
 /* Choose how to handle memory pools based on global symbol stuff.  */
 
 #if FFESYMBOL_globalCURRENT_ == FFESYMBOL_globalPROGUNIT_
 #define FFESYMBOL_SPACE_POOL_ ffe_pool_program_unit()
-#else
-#if FFESYMBOL_globalCURRENT_ == FFESYMBOL_globalFILE_
+#elif FFESYMBOL_globalCURRENT_ == FFESYMBOL_globalFILE_
 #define FFESYMBOL_SPACE_POOL_ ffe_pool_file()
-#endif
+#else
+#error
 #endif
 
 /* What kind of retraction is needed for a symbol?  */
@@ -244,7 +247,8 @@ ffesymbol_new_ (ffename n)
   s->generic = FFEINTRIN_genNONE;
   s->specific = FFEINTRIN_specNONE;
   s->implementation = FFEINTRIN_impNONE;
-  s->save = FALSE;
+  s->is_save = FALSE;
+  s->is_init = FALSE;
   s->do_iter = FALSE;
   s->reported = FALSE;
   s->explicit_where = FALSE;
@@ -340,7 +344,7 @@ ffesymbol_attrs_string (ffesymbolAttrs attrs)
     *--p = '\0';
   else
     sprintf (p, "?0x%" ffesymbolAttrs_f "x?", attrs);
-  assert ((p - &string[0]) < ARRAY_SIZE (string));
+  assert (((size_t) (p - &string[0])) < ARRAY_SIZE (string));
   return &string[0];
 }
 
@@ -798,27 +802,27 @@ ffesymbol_dump (ffesymbol s)
   assert (s != NULL);
 
   if (ffeinfo_size (s->info) != FFETARGET_charactersizeNONE)
-    fprintf (stdout, "%s:%d%s%s*%" ffetargetCharacterSize_f "u",
+    fprintf (dmpout, "%s:%d%s%s*%" ffetargetCharacterSize_f "u",
 	     ffesymbol_text (s),
 	     (int) ffeinfo_rank (s->info),
 	     ffeinfo_basictype_string (ffeinfo_basictype (s->info)),
 	     ffeinfo_kindtype_string (ffeinfo_kindtype (s->info)),
 	     ffeinfo_size (s->info));
   else
-    fprintf (stdout, "%s:%d%s%s",
+    fprintf (dmpout, "%s:%d%s%s",
 	     ffesymbol_text (s),
 	     (int) ffeinfo_rank (s->info),
 	     ffeinfo_basictype_string (ffeinfo_basictype (s->info)),
 	     ffeinfo_kindtype_string (ffeinfo_kindtype (s->info)));
   if ((k = ffeinfo_kind (s->info)) != FFEINFO_kindNONE)
-    fprintf (stdout, "/%s", ffeinfo_kind_string (k));
+    fprintf (dmpout, "/%s", ffeinfo_kind_string (k));
   if ((w = ffeinfo_where (s->info)) != FFEINFO_whereNONE)
-    fprintf (stdout, "@%s", ffeinfo_where_string (w));
+    fprintf (dmpout, "@%s", ffeinfo_where_string (w));
 
   if ((s->generic != FFEINTRIN_genNONE)
       || (s->specific != FFEINTRIN_specNONE)
       || (s->implementation != FFEINTRIN_impNONE))
-    fprintf (stdout, "{%s:%s:%s}",
+    fprintf (dmpout, "{%s:%s:%s}",
 	     ffeintrin_name_generic (s->generic),
 	     ffeintrin_name_specific (s->specific),
 	     ffeintrin_name_implementation (s->implementation));
@@ -935,7 +939,7 @@ ffesymbol_report (ffesymbol s)
   s->reported = TRUE;
 
   if (ffeinfo_size (s->info) != FFETARGET_charactersizeNONE)
-    fprintf (stdout, "\"%s\": %s %s %d%s%s*%" ffetargetCharacterSize_f "u",
+    fprintf (dmpout, "\"%s\": %s %s %d%s%s*%" ffetargetCharacterSize_f "u",
 	     ffesymbol_text (s),
 	     ffesymbol_state_string (s->state),
 	     ffesymbol_attrs_string (s->attrs),
@@ -944,7 +948,7 @@ ffesymbol_report (ffesymbol s)
 	     ffeinfo_kindtype_string (ffeinfo_kindtype (s->info)),
 	     ffeinfo_size (s->info));
   else
-    fprintf (stdout, "\"%s\": %s %s %d%s%s",
+    fprintf (dmpout, "\"%s\": %s %s %d%s%s",
 	     ffesymbol_text (s),
 	     ffesymbol_state_string (s->state),
 	     ffesymbol_attrs_string (s->attrs),
@@ -952,127 +956,132 @@ ffesymbol_report (ffesymbol s)
 	     ffeinfo_basictype_string (ffeinfo_basictype (s->info)),
 	     ffeinfo_kindtype_string (ffeinfo_kindtype (s->info)));
   if ((k = ffeinfo_kind (s->info)) != FFEINFO_kindNONE)
-    fprintf (stdout, "/%s", ffeinfo_kind_string (k));
+    fprintf (dmpout, "/%s", ffeinfo_kind_string (k));
   if ((w = ffeinfo_where (s->info)) != FFEINFO_whereNONE)
-    fprintf (stdout, "@%s", ffeinfo_where_string (w));
-  fputc ('\n', stdout);
+    fprintf (dmpout, "@%s", ffeinfo_where_string (w));
+  fputc ('\n', dmpout);
 
   if (s->dims != NULL)
     {
-      fprintf (stdout, "  dims: ");
+      fprintf (dmpout, "  dims: ");
       ffebld_dump (s->dims);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->extents != NULL)
     {
-      fprintf (stdout, "  extents: ");
+      fprintf (dmpout, "  extents: ");
       ffebld_dump (s->extents);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->dim_syms != NULL)
     {
-      fprintf (stdout, "  dim syms: ");
+      fprintf (dmpout, "  dim syms: ");
       ffebld_dump (s->dim_syms);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->array_size != NULL)
     {
-      fprintf (stdout, "  array size: ");
+      fprintf (dmpout, "  array size: ");
       ffebld_dump (s->array_size);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->init != NULL)
     {
-      fprintf (stdout, "  init-value: ");
+      fprintf (dmpout, "  init-value: ");
       if (ffebld_op (s->init) == FFEBLD_opANY)
-	fputs ("<any>\n", stdout);
+	fputs ("<any>\n", dmpout);
       else
 	{
 	  ffebld_dump (s->init);
-	  fputs ("\n", stdout);
+	  fputs ("\n", dmpout);
 	}
     }
 
   if (s->accretion != NULL)
     {
-      fprintf (stdout, "  accretion (%" ffetargetOffset_f "u left): ",
+      fprintf (dmpout, "  accretion (%" ffetargetOffset_f "u left): ",
 	       s->accretes);
       ffebld_dump (s->accretion);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
   else if (s->accretes != 0)
-    fprintf (stdout, "  accretes!! = %" ffetargetOffset_f "u left\n",
+    fprintf (dmpout, "  accretes!! = %" ffetargetOffset_f "u left\n",
 	     s->accretes);
 
   if (s->dummy_args != NULL)
     {
-      fprintf (stdout, "  dummies: ");
+      fprintf (dmpout, "  dummies: ");
       ffebld_dump (s->dummy_args);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->namelist != NULL)
     {
-      fprintf (stdout, "  namelist: ");
+      fprintf (dmpout, "  namelist: ");
       ffebld_dump (s->namelist);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->common_list != NULL)
     {
-      fprintf (stdout, "  common-list: ");
+      fprintf (dmpout, "  common-list: ");
       ffebld_dump (s->common_list);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->sfunc_expr != NULL)
     {
-      fprintf (stdout, "  sfunc expression: ");
+      fprintf (dmpout, "  sfunc expression: ");
       ffebld_dump (s->sfunc_expr);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
-  if (s->save)
+  if (s->is_save)
     {
-      fprintf (stdout, "  SAVEd\n");
+      fprintf (dmpout, "  SAVEd\n");
+    }
+
+  if (s->is_init)
+    {
+      fprintf (dmpout, "  initialized\n");
     }
 
   if (s->do_iter)
     {
-      fprintf (stdout, "  DO-loop iteration variable (currently)\n");
+      fprintf (dmpout, "  DO-loop iteration variable (currently)\n");
     }
 
   if (s->explicit_where)
     {
-      fprintf (stdout, "  Explicit INTRINSIC/EXTERNAL\n");
+      fprintf (dmpout, "  Explicit INTRINSIC/EXTERNAL\n");
     }
 
   if (s->namelisted)
     {
-      fprintf (stdout, "  Namelisted\n");
+      fprintf (dmpout, "  Namelisted\n");
     }
 
   if (s->common != NULL)
     {
-      fprintf (stdout, "  COMMON area: %s\n", ffesymbol_text (s->common));
+      fprintf (dmpout, "  COMMON area: %s\n", ffesymbol_text (s->common));
     }
 
   if (s->equiv != NULL)
     {
-      fprintf (stdout, "  EQUIVALENCE information: ");
+      fprintf (dmpout, "  EQUIVALENCE information: ");
       ffeequiv_dump (s->equiv);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   if (s->storage != NULL)
     {
-      fprintf (stdout, "  Storage: ");
+      fprintf (dmpout, "  Storage: ");
       ffestorag_dump (s->storage);
-      fputs ("\n", stdout);
+      fputs ("\n", dmpout);
     }
 
   return s;
@@ -1268,7 +1277,7 @@ ffesymbol_terminate_1 ()
   ffename_space_kill (ffesymbol_global_);
   ffesymbol_global_ = NULL;
 
-  ffesymbol_kill_manifest ();
+  ffesymbol_kill_manifest_ ();
 #endif
 }
 
@@ -1303,6 +1312,42 @@ ffesymbol_terminate_4 ()
   ffesymbol_sfunc_ = NULL;
 }
 
+/* Update INIT info to TRUE and all equiv/storage too.
+
+   If INIT flag is TRUE, does nothing.	Else sets it to TRUE and calls
+   on the ffeequiv and ffestorag modules to update their INIT flags if
+   the <s> symbol has those objects, and also updates the common area if
+   it exists.  */
+
+void
+ffesymbol_update_init (ffesymbol s)
+{
+  ffebld item;
+
+  if (s->is_init)
+    return;
+
+  s->is_init = TRUE;
+
+  if ((s->equiv != NULL)
+      && !ffeequiv_is_init (s->equiv))
+    ffeequiv_update_init (s->equiv);
+
+  if ((s->storage != NULL)
+      && !ffestorag_is_init (s->storage))
+    ffestorag_update_init (s->storage);
+
+  if ((s->common != NULL)
+      && (!ffesymbol_is_init (s->common)))
+    ffesymbol_update_init (s->common);
+
+  for (item = s->common_list; item != NULL; item = ffebld_trail (item))
+    {
+      if (!ffesymbol_is_init (ffebld_symter (ffebld_head (item))))
+	ffesymbol_update_init (ffebld_symter (ffebld_head (item)));
+    }
+}
+
 /* Update SAVE info to TRUE and all equiv/storage too.
 
    If SAVE flag is TRUE, does nothing.	Else sets it to TRUE and calls
@@ -1315,20 +1360,26 @@ ffesymbol_update_save (ffesymbol s)
 {
   ffebld item;
 
-  if (s->save)
+  if (s->is_save)
     return;
 
-  s->save = TRUE;
+  s->is_save = TRUE;
 
-  if (s->equiv != NULL)
+  if ((s->equiv != NULL)
+      && !ffeequiv_is_save (s->equiv))
     ffeequiv_update_save (s->equiv);
 
-  if (s->storage != NULL)
+  if ((s->storage != NULL)
+      && !ffestorag_is_save (s->storage))
     ffestorag_update_save (s->storage);
 
-  if (s->common != NULL)
+  if ((s->common != NULL)
+      && (!ffesymbol_is_save (s->common)))
     ffesymbol_update_save (s->common);
 
   for (item = s->common_list; item != NULL; item = ffebld_trail (item))
-    ffesymbol_update_save (ffebld_symter (ffebld_head (item)));
+    {
+      if (!ffesymbol_is_save (ffebld_symter (ffebld_head (item))))
+	ffesymbol_update_save (ffebld_symter (ffebld_head (item)));
+    }
 }
