@@ -415,8 +415,32 @@ ffetarget_gt_character1 (bool *res, ffetargetCharacter1 l,
   *res = (memcmp (l.text, r.text, l.length) > 0);
   return FFEBAD;
 }
-
 #endif
+
+#if FFETARGET_okCHARACTER1
+bool
+ffetarget_iszero_character1 (ffetargetCharacter1 constant)
+{
+  ffetargetCharacterSize i;
+
+  for (i = 0; i < constant.length; ++i)
+    if (constant.text[i] != 0)
+      return FALSE;
+  return TRUE;
+}
+#endif
+
+bool
+ffetarget_iszero_hollerith (ffetargetHollerith constant)
+{
+  ffetargetHollerithSize i;
+
+  for (i = 0; i < constant.length; ++i)
+    if (constant.text[i] != 0)
+      return FALSE;
+  return TRUE;
+}
+
 /* ffetarget_layout -- Do storage requirement analysis for entity
 
    Return the alignment/modulo requirements along with the size, given the
@@ -707,9 +731,9 @@ ffetarget_convert_character1_hollerith (ffetargetCharacter1 *res,
 
 #if FFETARGET_okCHARACTER1
 ffebad
-ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
+ffetarget_convert_character1_integer4 (ffetargetCharacter1 *res,
 				       ffetargetCharacterSize size,
-				       ffetargetInteger1 l, mallocPool pool)
+				       ffetargetInteger4 l, mallocPool pool)
 {
   long long int l1;
   long int l2;
@@ -793,9 +817,9 @@ ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
 
 #if FFETARGET_okCHARACTER1
 ffebad
-ffetarget_convert_character1_logical1 (ffetargetCharacter1 *res,
+ffetarget_convert_character1_logical4 (ffetargetCharacter1 *res,
 				       ffetargetCharacterSize size,
-				       ffetargetLogical1 l, mallocPool pool)
+				       ffetargetLogical4 l, mallocPool pool)
 {
   long long int l1;
   long int l2;
@@ -1397,7 +1421,7 @@ ffetarget_power_complexdefault_integerdefault (ffetargetComplexDefault *res,
     {
       ffetarget_real1_zero (&res->real);
       ffetarget_real1_zero (&res->imaginary);
-      return FFEBAD_BAD_POWER;
+      return FFEBAD;
     }
 
   if (r == 0)
@@ -1523,7 +1547,7 @@ ffetarget_power_complexdouble_integerdefault (ffetargetComplexDouble *res,
     {
       ffetarget_real2_zero (&res->real);
       ffetarget_real2_zero (&res->imaginary);
-      return FFEBAD_BAD_POWER;
+      return FFEBAD;
     }
 
   if (r == 0)
@@ -1641,7 +1665,7 @@ ffetarget_power_integerdefault_integerdefault (ffetargetIntegerDefault *res,
   if (l == 0)
     {
       *res = 0;
-      return FFEBAD_BAD_POWER;
+      return FFEBAD;
     }
 
   if (r == 0)
@@ -1652,7 +1676,14 @@ ffetarget_power_integerdefault_integerdefault (ffetargetIntegerDefault *res,
 
   if (r < 0)
     {
-      *res = (l == 1) ? 1 : 0;
+      if (l == 1)
+	*res = 1;
+      else if (l == 0)
+	*res = 1;
+      else if (l == -1)
+	*res = ((-r) & 1) == 0 ? 1 : -1;
+      else
+	*res = 0;
       return FFEBAD;
     }
 
@@ -1689,7 +1720,7 @@ ffetarget_power_realdefault_integerdefault (ffetargetRealDefault *res,
   if (ffetarget_iszero_real1 (l))
     {
       ffetarget_real1_zero (res);
-      return FFEBAD_BAD_POWER;
+      return FFEBAD;
     }
 
   if (r == 0)
@@ -1751,7 +1782,7 @@ ffetarget_power_realdouble_integerdefault (ffetargetRealDouble *res,
   if (ffetarget_iszero_real2 (l))
     {
       ffetarget_real2_zero (res);
-      return FFEBAD_BAD_POWER;
+      return FFEBAD;
     }
 
   if (r == 0)
@@ -1802,7 +1833,7 @@ ffetarget_power_realdouble_integerdefault (ffetargetRealDouble *res,
 /* ffetarget_print_binary -- Output typeless binary integer
 
    ffetargetTypeless val;
-   ffetarget_typeless_binary(stdout,val);  */
+   ffetarget_typeless_binary(dmpout,val);  */
 
 void
 ffetarget_print_binary (FILE *f, ffetargetTypeless value)
@@ -1811,7 +1842,7 @@ ffetarget_print_binary (FILE *f, ffetargetTypeless value)
   char digits[sizeof (value) * CHAR_BIT + 1];
 
   if (f == NULL)
-    f = stdout;
+    f = dmpout;
 
   p = &digits[ARRAY_SIZE (digits) - 1];
   *p = '\0';
@@ -1827,7 +1858,7 @@ ffetarget_print_binary (FILE *f, ffetargetTypeless value)
 /* ffetarget_print_character1 -- Output character string
 
    ffetargetCharacter1 val;
-   ffetarget_print_character1(stdout,val);  */
+   ffetarget_print_character1(dmpout,val);  */
 
 void
 ffetarget_print_character1 (FILE *f, ffetargetCharacter1 value)
@@ -1835,16 +1866,16 @@ ffetarget_print_character1 (FILE *f, ffetargetCharacter1 value)
   unsigned char *p;
   ffetargetCharacterSize i;
 
-  fputc ('\'', stdout);
+  fputc ('\'', dmpout);
   for (i = 0, p = value.text; i < value.length; ++i, ++p)
     ffetarget_print_char_ (f, *p);
-  fputc ('\'', stdout);
+  fputc ('\'', dmpout);
 }
 
 /* ffetarget_print_hollerith -- Output hollerith string
 
    ffetargetHollerith val;
-   ffetarget_print_hollerith(stdout,val);  */
+   ffetarget_print_hollerith(dmpout,val);  */
 
 void
 ffetarget_print_hollerith (FILE *f, ffetargetHollerith value)
@@ -1852,16 +1883,16 @@ ffetarget_print_hollerith (FILE *f, ffetargetHollerith value)
   unsigned char *p;
   ffetargetHollerithSize i;
 
-  fputc ('\'', stdout);
+  fputc ('\'', dmpout);
   for (i = 0, p = value.text; i < value.length; ++i, ++p)
     ffetarget_print_char_ (f, *p);
-  fputc ('\'', stdout);
+  fputc ('\'', dmpout);
 }
 
 /* ffetarget_print_octal -- Output typeless octal integer
 
    ffetargetTypeless val;
-   ffetarget_print_octal(stdout,val);  */
+   ffetarget_print_octal(dmpout,val);  */
 
 void
 ffetarget_print_octal (FILE *f, ffetargetTypeless value)
@@ -1870,7 +1901,7 @@ ffetarget_print_octal (FILE *f, ffetargetTypeless value)
   char digits[sizeof (value) * CHAR_BIT / 3 + 1];
 
   if (f == NULL)
-    f = stdout;
+    f = dmpout;
 
   p = &digits[ARRAY_SIZE (digits) - 3];
   *p = '\0';
@@ -1886,7 +1917,7 @@ ffetarget_print_octal (FILE *f, ffetargetTypeless value)
 /* ffetarget_print_hex -- Output typeless hex integer
 
    ffetargetTypeless val;
-   ffetarget_print_hex(stdout,val);  */
+   ffetarget_print_hex(dmpout,val);  */
 
 void
 ffetarget_print_hex (FILE *f, ffetargetTypeless value)
@@ -1896,7 +1927,7 @@ ffetarget_print_hex (FILE *f, ffetargetTypeless value)
   static char hexdigits[16] = "0123456789ABCDEF";
 
   if (f == NULL)
-    f = stdout;
+    f = dmpout;
 
   p = &digits[ARRAY_SIZE (digits) - 3];
   *p = '\0';
