@@ -141,11 +141,7 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 /* 1 if external Fortran names with underscores already in them should
    have an extra underscore appended (in addition to the one they
-   might already have appened if FFETARGET_defaultEXTERNAL_UNDERSCORED).
-   NOTE: See gcc/f/com.c for other changes that need to be made
-   if this is not #define'd to be 1, but underscoring is turned on.
-   Right now there's only the option to disable underscoring in general,
-   not to change just this setting.  */
+   might already have appened if FFETARGET_defaultEXTERNAL_UNDERSCORED). */
 
 #ifndef FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED
 #define FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED 1
@@ -222,13 +218,13 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #endif
 
 #define FFETARGET_okINTEGER1 1
-#define FFETARGET_okINTEGER2 0
-#define FFETARGET_okINTEGER3 0
-#define FFETARGET_okINTEGER4 0
+#define FFETARGET_okINTEGER2 1
+#define FFETARGET_okINTEGER3 1
+#define FFETARGET_okINTEGER4 1
 #define FFETARGET_okLOGICAL1 1
-#define FFETARGET_okLOGICAL2 0
-#define FFETARGET_okLOGICAL3 0
-#define FFETARGET_okLOGICAL4 0
+#define FFETARGET_okLOGICAL2 1
+#define FFETARGET_okLOGICAL3 1
+#define FFETARGET_okLOGICAL4 1
 #define FFETARGET_okREAL1 1
 #define FFETARGET_okREAL2 1
 #define FFETARGET_okREAL3 0
@@ -282,8 +278,7 @@ typedef short int ffetargetInteger3;
 #endif
 #if FFETARGET_okINTEGER4
 typedef long long int ffetargetInteger4;
-#define ffetargetInteger4_f
-?
+#define ffetargetInteger4_f "ll"
 #endif
 #if FFETARGET_okINTEGER5
 typedef ? ffetargetInteger5;
@@ -319,8 +314,7 @@ typedef short int ffetargetLogical3;
 #endif
 #if FFETARGET_okLOGICAL4
 typedef long long int ffetargetLogical4;
-#define ffetargetLogical4_f
-?
+#define ffetargetLogical4_f "ll"
 #endif
 #if FFETARGET_okLOGICAL5
 typedef ? ffetargetLogical5;
@@ -591,13 +585,13 @@ ffebad ffetarget_convert_character1_hollerith (ffetargetCharacter1 *res,
 					    ffetargetCharacterSize res_size,
 					       ffetargetHollerith l,
 					       mallocPool pool);
-ffebad ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
+ffebad ffetarget_convert_character1_integer4 (ffetargetCharacter1 *res,
 					      ffetargetCharacterSize res_size,
-					      ffetargetInteger1 l,
+					      ffetargetInteger4 l,
 					      mallocPool pool);
-ffebad ffetarget_convert_character1_logical1 (ffetargetCharacter1 *res,
+ffebad ffetarget_convert_character1_logical4 (ffetargetCharacter1 *res,
 					      ffetargetCharacterSize res_size,
-					      ffetargetLogical1 l,
+					      ffetargetLogical4 l,
 					      mallocPool pool);
 ffebad ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
 					    ffetargetCharacterSize res_size,
@@ -698,6 +692,10 @@ void ffetarget_integer_bad_magical_precedence (ffelexToken integer,
 void ffetarget_integer_bad_magical_precedence_binary (ffelexToken integer,
 						      ffelexToken minus,
 						      ffelexToken higher_op);
+#if FFETARGET_okCHARACTER1
+bool ffetarget_iszero_character1 (ffetargetCharacter1 constant);
+#endif
+bool ffetarget_iszero_hollerith (ffetargetHollerith constant);
 void ffetarget_layout (char *error_text, ffetargetAlign *alignment,
 		       ffetargetAlign *modulo, ffetargetOffset *size,
 		       ffeinfoBasictype bt, ffeinfoKindtype kt,
@@ -856,6 +854,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_add_integer1(res,l,r) (*(res) = (l) + (r), FFEBAD)
 #define ffetarget_add_integer2(res,l,r) (*(res) = (l) + (r), FFEBAD)
 #define ffetarget_add_integer3(res,l,r) (*(res) = (l) + (r), FFEBAD)
+#define ffetarget_add_integer4(res,l,r) (*(res) = (l) + (r), FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_add_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr, resr; \
@@ -880,22 +879,38 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_and_integer1(res,l,r) (*(res) = (l) & (r), FFEBAD)
 #define ffetarget_and_integer2(res,l,r) (*(res) = (l) & (r), FFEBAD)
 #define ffetarget_and_integer3(res,l,r) (*(res) = (l) & (r), FFEBAD)
+#define ffetarget_and_integer4(res,l,r) (*(res) = (l) & (r), FFEBAD)
 #define ffetarget_and_logical1(res,l,r) (*(res) = (l) && (r), FFEBAD)
 #define ffetarget_and_logical2(res,l,r) (*(res) = (l) && (r), FFEBAD)
 #define ffetarget_and_logical3(res,l,r) (*(res) = (l) && (r), FFEBAD)
+#define ffetarget_and_logical4(res,l,r) (*(res) = (l) && (r), FFEBAD)
 #define ffetarget_binarymil(v,t) ffetarget_typeless_binary (v, t)
 #define ffetarget_binaryvxt(v,t) ffetarget_typeless_binary (v, t)
 #define ffetarget_cmp_integer1(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_integer2(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_integer3(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
+#define ffetarget_cmp_integer4(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_logical1(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_logical2(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_logical3(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
+#define ffetarget_cmp_logical4(l,r) ((l) == (r) ? 0 : ((l) < (r) ? -1 : 1))
 #define ffetarget_cmp_real1(l,r) memcmp (&(l), &(r), sizeof(l))
 #define ffetarget_cmp_real2(l,r) memcmp (&(l), &(r), sizeof(l))
 #define ffetarget_cmp_real3(l,r) memcmp (&(l), &(r), sizeof(l))
 #define ffetarget_cmp_typeless(l,r) \
   memcmp (&(l), &(r), sizeof ((l)))
+#define ffetarget_convert_character1_integer1(res,res_size,l,pool) \
+        ffetarget_convert_character1_integer4(res,res_size,(ffetargetInteger4)l,pool)
+#define ffetarget_convert_character1_integer2(res,res_size,l,pool) \
+        ffetarget_convert_character1_integer4(res,res_size,(ffetargetInteger4)l,pool)
+#define ffetarget_convert_character1_integer3(res,res_size,l,pool) \
+        ffetarget_convert_character1_integer4(res,res_size,(ffetargetInteger4)l,pool)
+#define ffetarget_convert_character1_logical1(res,res_size,l,pool) \
+        ffetarget_convert_character1_logical4(res,res_size,(ffetargetLogical4)l,pool)
+#define ffetarget_convert_character1_logical2(res,res_size,l,pool) \
+        ffetarget_convert_character1_logical4(res,res_size,(ffetargetLogical4)l,pool)
+#define ffetarget_convert_character1_logical3(res,res_size,l,pool) \
+        ffetarget_convert_character1_logical4(res,res_size,(ffetargetLogical4)l,pool)
 #define ffetarget_convert_complex1_character1(res,l) \
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_complex1_hollerith(res,l) \
@@ -928,6 +943,9 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ((res)->real = (l), (res)->imaginary = 0, FFEBAD)
 #endif
 #define ffetarget_convert_complex1_integer1 ffetarget_convert_complex1_integer
+#define ffetarget_convert_complex1_integer2 ffetarget_convert_complex1_integer
+#define ffetarget_convert_complex1_integer3 ffetarget_convert_complex1_integer
+#define ffetarget_convert_complex1_integer4 ffetarget_convert_complex1_integer
 #ifdef REAL_ARITHMETIC
 #define ffetarget_convert_complex1_real1(res,l) \
   ((res)->real = (l), \
@@ -977,6 +995,9 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ((res)->real = (l), (res)->imaginary = 0, FFEBAD)
 #endif
 #define ffetarget_convert_complex2_integer1 ffetarget_convert_complex2_integer
+#define ffetarget_convert_complex2_integer2 ffetarget_convert_complex2_integer
+#define ffetarget_convert_complex2_integer3 ffetarget_convert_complex2_integer
+#define ffetarget_convert_complex2_integer4 ffetarget_convert_complex2_integer
 #ifdef REAL_ARITHMETIC
 #define ffetarget_convert_complex2_real1(res,l) \
   ({ REAL_VALUE_TYPE lr; \
@@ -994,6 +1015,81 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_convert_complex2_real2(res,l) \
   ((res)->real = (l), (res)->imaginary = 0, FFEBAD)
 #endif
+#define ffetarget_convert_integer2_character1(res,l) \
+        ffetarget_convert_integer1_character1(res,l)
+#define ffetarget_convert_integer2_complex1(res,l) \
+        ffetarget_convert_integer1_complex1(res,l)
+#define ffetarget_convert_integer2_complex2(res,l) \
+        ffetarget_convert_integer1_complex2(res,l)
+#define ffetarget_convert_integer2_hollerith(res,l) \
+        ffetarget_convert_integer1_hollerith(res,l)
+#define ffetarget_convert_integer2_integer1(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer2_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer2_integer4(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer2_logical1(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer2_logical2(res,l) \
+        ffetarget_convert_integer2_logical1(res,l)
+#define ffetarget_convert_integer2_logical3(res,l) \
+        ffetarget_convert_integer2_logical1(res,l)
+#define ffetarget_convert_integer2_logical4(res,l) \
+        ffetarget_convert_integer2_logical1(res,l)
+#define ffetarget_convert_integer2_real1(res,l) \
+        ffetarget_convert_integer1_real1(res,l)
+#define ffetarget_convert_integer2_real2(res,l) \
+        ffetarget_convert_integer1_real2(res,l)
+#define ffetarget_convert_integer2_typeless(res,l) \
+        ffetarget_convert_integer1_typeless(res,l)
+#define ffetarget_convert_integer3_character1(res,l) \
+        ffetarget_convert_integer1_character1(res,l)
+#define ffetarget_convert_integer3_complex1(res,l) \
+        ffetarget_convert_integer1_complex1(res,l)
+#define ffetarget_convert_integer3_complex2(res,l) \
+        ffetarget_convert_integer1_complex2(res,l)
+#define ffetarget_convert_integer3_hollerith(res,l) \
+        ffetarget_convert_integer1_hollerith(res,l)
+#define ffetarget_convert_integer3_integer1(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer3_integer2(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer3_integer4(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer3_logical1(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer3_logical2(res,l) \
+        ffetarget_convert_integer3_logical1(res,l)
+#define ffetarget_convert_integer3_logical3(res,l) \
+        ffetarget_convert_integer3_logical1(res,l)
+#define ffetarget_convert_integer3_logical4(res,l) \
+        ffetarget_convert_integer3_logical1(res,l)
+#define ffetarget_convert_integer3_real1(res,l) \
+        ffetarget_convert_integer1_real1(res,l)
+#define ffetarget_convert_integer3_real2(res,l) \
+        ffetarget_convert_integer1_real2(res,l)
+#define ffetarget_convert_integer3_typeless(res,l) \
+        ffetarget_convert_integer1_typeless(res,l)
+#define ffetarget_convert_integer4_character1(res,l) \
+        ffetarget_convert_integer1_character1(res,l)
+#define ffetarget_convert_integer4_complex1(res,l) \
+        ffetarget_convert_integer1_complex1(res,l)
+#define ffetarget_convert_integer4_complex2(res,l) \
+        ffetarget_convert_integer1_complex2(res,l)
+#define ffetarget_convert_integer4_hollerith(res,l) \
+        ffetarget_convert_integer1_hollerith(res,l)
+#define ffetarget_convert_integer4_integer1(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer4_integer2(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer4_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer4_logical1(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer4_logical2(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer4_logical3(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer4_logical4(res,l) \
+        ffetarget_convert_integer1_logical1(res,l)
+#define ffetarget_convert_integer4_real1(res,l) \
+        ffetarget_convert_integer1_real1(res,l)
+#define ffetarget_convert_integer4_real2(res,l) \
+        ffetarget_convert_integer1_real2(res,l)
+#define ffetarget_convert_integer4_typeless(res,l) \
+        ffetarget_convert_integer1_typeless(res,l)
 #define ffetarget_convert_logical1_character1(res,l) \
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical1_hollerith(res,l) \
@@ -1002,9 +1098,11 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical1_logical2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_logical3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical1_logical4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical1_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical1_integer4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_character1(res,l) \
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical2_hollerith(res,l) \
@@ -1013,9 +1111,11 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical2_logical1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_logical3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical2_logical4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical2_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical2_integer4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_character1(res,l) \
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical3_hollerith(res,l) \
@@ -1024,9 +1124,24 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_logical3_logical1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_logical2(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical3_logical4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_integer1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_logical3_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical3_integer4(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_character1(res,l) \
+  ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
+#define ffetarget_convert_logical4_hollerith(res,l) \
+  ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
+#define ffetarget_convert_logical4_typeless(res,l) \
+  ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
+#define ffetarget_convert_logical4_logical1(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_logical2(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_logical3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_integer1(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_integer2(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_logical4_integer4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_integer1_character1(res,l) \
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_integer1_hollerith(res,l) \
@@ -1035,9 +1150,11 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_integer1_integer2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_integer1_integer3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer1_integer4(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_integer1_logical1(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_integer1_logical2(res,l) (*(res) = (l), FFEBAD)
 #define ffetarget_convert_integer1_logical3(res,l) (*(res) = (l), FFEBAD)
+#define ffetarget_convert_integer1_logical4(res,l) (*(res) = (l), FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_convert_integer1_real1(res,l) \
   ({ REAL_VALUE_TYPE lr; \
@@ -1073,6 +1190,12 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real1_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
+#define ffetarget_convert_real1_integer2(res,l) \
+        ffetarget_convert_real1_integer1(res,l)
+#define ffetarget_convert_real1_integer3(res,l) \
+        ffetarget_convert_real1_integer1(res,l)
+#define ffetarget_convert_real1_integer4(res,l) \
+        ffetarget_convert_real1_integer1(res,l)
 #define ffetarget_convert_real1_typeless(res,l) \
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real1_complex1(res,l) (*(res) = (l).real, FFEBAD)
@@ -1101,6 +1224,12 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   ffetarget_convert_any_character1_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real2_hollerith(res,l) \
   ffetarget_convert_any_hollerith_ ((char *) (res), sizeof(*(res)), l)
+#define ffetarget_convert_real2_integer2(res,l) \
+        ffetarget_convert_real2_integer1(res,l)
+#define ffetarget_convert_real2_integer3(res,l) \
+        ffetarget_convert_real2_integer1(res,l)
+#define ffetarget_convert_real2_integer4(res,l) \
+        ffetarget_convert_real2_integer1(res,l)
 #define ffetarget_convert_real2_typeless(res,l) \
   ffetarget_convert_any_typeless_ ((char *) (res), sizeof(*(res)), l)
 #define ffetarget_convert_real2_complex1(res,l) \
@@ -1129,6 +1258,12 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_divide_integer1(res,l,r) \
   (((r) == 0) ? (*(res) = 0, FFEBAD_DIV_BY_ZERO)  \
    : (*(res) = (l) / (r), FFEBAD))
+#define ffetarget_divide_integer2(res,l,r) \
+        ffetarget_divide_integer1(res,l,r)
+#define ffetarget_divide_integer3(res,l,r) \
+        ffetarget_divide_integer1(res,l,r)
+#define ffetarget_divide_integer4(res,l,r) \
+        ffetarget_divide_integer1(res,l,r)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_divide_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr, resr; \
@@ -1197,6 +1332,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   (*(res) = ((l) == (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_eq_integer3(res,l,r) \
   (*(res) = ((l) == (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_eq_integer4(res,l,r) \
+  (*(res) = ((l) == (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_eq_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1219,15 +1356,19 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_eqv_integer1(res,l,r) (*(res) = (l) ^ ~(r), FFEBAD)
 #define ffetarget_eqv_integer2(res,l,r) (*(res) = (l) ^ ~(r), FFEBAD)
 #define ffetarget_eqv_integer3(res,l,r) (*(res) = (l) ^ ~(r), FFEBAD)
-#define ffetarget_eqv_logical1(res,l,r) (*(res) = !!(l) == !!(r), FFEBAD)
-#define ffetarget_eqv_logical2(res,l,r) (*(res) = !!(l) == !!(r), FFEBAD)
-#define ffetarget_eqv_logical3(res,l,r) (*(res) = !!(l) == !!(r), FFEBAD)
+#define ffetarget_eqv_integer4(res,l,r) (*(res) = (l) ^ ~(r), FFEBAD)
+#define ffetarget_eqv_logical1(res,l,r) (*(res) = (l) == (r), FFEBAD)
+#define ffetarget_eqv_logical2(res,l,r) (*(res) = (l) == (r), FFEBAD)
+#define ffetarget_eqv_logical3(res,l,r) (*(res) = (l) == (r), FFEBAD)
+#define ffetarget_eqv_logical4(res,l,r) (*(res) = (l) == (r), FFEBAD)
 #define ffetarget_ge_integer1(res,l,r) \
   (*(res) = ((l) >= (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_ge_integer2(res,l,r) \
   (*(res) = ((l) >= (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_ge_integer3(res,l,r) \
   (*(res) = ((l) >= (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_ge_integer4(res,l,r) \
+  (*(res) = ((l) >= (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_ge_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1253,6 +1394,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   (*(res) = ((l) > (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_gt_integer3(res,l,r) \
   (*(res) = ((l) > (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_gt_integer4(res,l,r) \
+  (*(res) = ((l) > (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_gt_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1300,6 +1443,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_iszero_real1(l) ((l) == 0.)
 #define ffetarget_iszero_real2(l) ((l) == 0.)
 #endif
+#define ffetarget_iszero_typeless(l) ((l) == 0)
 #define ffetarget_logical1(v,truth) (*(v) = truth ? 1 : 0)
 #define ffetarget_le_integer1(res,l,r) \
   (*(res) = ((l) <= (r)) ? TRUE : FALSE, FFEBAD)
@@ -1307,6 +1451,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   (*(res) = ((l) <= (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_le_integer3(res,l,r) \
   (*(res) = ((l) <= (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_le_integer4(res,l,r) \
+  (*(res) = ((l) <= (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_le_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1334,6 +1480,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   (*(res) = ((l) < (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_lt_integer3(res,l,r) \
   (*(res) = ((l) < (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_lt_integer4(res,l,r) \
+  (*(res) = ((l) < (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_lt_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1367,6 +1515,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_multiply_integer1(res,l,r) (*(res) = (l) * (r), FFEBAD)
 #define ffetarget_multiply_integer2(res,l,r) (*(res) = (l) * (r), FFEBAD)
 #define ffetarget_multiply_integer3(res,l,r) (*(res) = (l) * (r), FFEBAD)
+#define ffetarget_multiply_integer4(res,l,r) (*(res) = (l) * (r), FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_multiply_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr, resr; \
@@ -1419,6 +1568,8 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   (*(res) = ((l) != (r)) ? TRUE : FALSE, FFEBAD)
 #define ffetarget_ne_integer3(res,l,r) \
   (*(res) = ((l) != (r)) ? TRUE : FALSE, FFEBAD)
+#define ffetarget_ne_integer4(res,l,r) \
+  (*(res) = ((l) != (r)) ? TRUE : FALSE, FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_ne_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr; \
@@ -1441,15 +1592,19 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_neqv_integer1(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
 #define ffetarget_neqv_integer2(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
 #define ffetarget_neqv_integer3(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
-#define ffetarget_neqv_logical1(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
-#define ffetarget_neqv_logical2(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
-#define ffetarget_neqv_logical3(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
+#define ffetarget_neqv_integer4(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
+#define ffetarget_neqv_logical1(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_neqv_logical2(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_neqv_logical3(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_neqv_logical4(res,l,r) (*(res) = (l) != (r), FFEBAD)
 #define ffetarget_not_integer1(res,l) (*(res) = ~(l), FFEBAD)
 #define ffetarget_not_integer2(res,l) (*(res) = ~(l), FFEBAD)
 #define ffetarget_not_integer3(res,l) (*(res) = ~(l), FFEBAD)
+#define ffetarget_not_integer4(res,l) (*(res) = ~(l), FFEBAD)
 #define ffetarget_not_logical1(res,l) (*(res) = !(l), FFEBAD)
 #define ffetarget_not_logical2(res,l) (*(res) = !(l), FFEBAD)
 #define ffetarget_not_logical3(res,l) (*(res) = !(l), FFEBAD)
+#define ffetarget_not_logical4(res,l) (*(res) = !(l), FFEBAD)
 #define ffetarget_octalmil(v,t) ffetarget_typeless_octal (v, t)
 #define ffetarget_octalvxt(v,t) ffetarget_typeless_octal (v, t)
 #define ffetarget_offset(res,l) (*(res) = (l), TRUE)	/* Overflow? */
@@ -1460,9 +1615,11 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_or_integer1(res,l,r) (*(res) = (l) | (r), FFEBAD)
 #define ffetarget_or_integer2(res,l,r) (*(res) = (l) | (r), FFEBAD)
 #define ffetarget_or_integer3(res,l,r) (*(res) = (l) | (r), FFEBAD)
+#define ffetarget_or_integer4(res,l,r) (*(res) = (l) | (r), FFEBAD)
 #define ffetarget_or_logical1(res,l,r) (*(res) = (l) || (r), FFEBAD)
 #define ffetarget_or_logical2(res,l,r) (*(res) = (l) || (r), FFEBAD)
 #define ffetarget_or_logical3(res,l,r) (*(res) = (l) || (r), FFEBAD)
+#define ffetarget_or_logical4(res,l,r) (*(res) = (l) || (r), FFEBAD)
 #define ffetarget_print_binarymil(f,v) ffetarget_print_binary (f, v)
 #define ffetarget_print_binaryvxt(f,v) ffetarget_print_binary (f, v)
 #define ffetarget_print_hexxmil(f,v) ffetarget_print_hex (f, v)
@@ -1475,12 +1632,16 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
   fprintf ((f), "%" ffetargetInteger2_f "d", (v))
 #define ffetarget_print_integer3(f,v) \
   fprintf ((f), "%" ffetargetInteger3_f "d", (v))
+#define ffetarget_print_integer4(f,v) \
+  fprintf ((f), "%" ffetargetInteger4_f "d", (v))
 #define ffetarget_print_logical1(f,v) \
   fprintf ((f), "%" ffetargetLogical1_f "d", (v))
 #define ffetarget_print_logical2(f,v) \
   fprintf ((f), "%" ffetargetLogical2_f "d", (v))
 #define ffetarget_print_logical3(f,v) \
   fprintf ((f), "%" ffetargetLogical3_f "d", (v))
+#define ffetarget_print_logical4(f,v) \
+  fprintf ((f), "%" ffetargetLogical4_f "d", (v))
 #define ffetarget_print_octalmil(f,v) ffetarget_print_octal(f,v)
 #define ffetarget_print_octalvxt(f,v) ffetarget_print_octal(f,v)
 #ifdef REAL_ARITHMETIC
@@ -1561,6 +1722,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_subtract_integer1(res,l,r) (*(res) = (l) - (r), FFEBAD)
 #define ffetarget_subtract_integer2(res,l,r) (*(res) = (l) - (r), FFEBAD)
 #define ffetarget_subtract_integer3(res,l,r) (*(res) = (l) - (r), FFEBAD)
+#define ffetarget_subtract_integer4(res,l,r) (*(res) = (l) - (r), FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_subtract_real1(res,l,r) \
   ({ REAL_VALUE_TYPE lr, rr, resr; \
@@ -1615,6 +1777,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_uminus_integer1(res,l) (*(res) = -(l), FFEBAD)
 #define ffetarget_uminus_integer2(res,l) (*(res) = -(l), FFEBAD)
 #define ffetarget_uminus_integer3(res,l) (*(res) = -(l), FFEBAD)
+#define ffetarget_uminus_integer4(res,l) (*(res) = -(l), FFEBAD)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_uminus_real1(res,l) \
   ({ REAL_VALUE_TYPE lr, resr; \
@@ -1642,9 +1805,11 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_xor_integer1(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
 #define ffetarget_xor_integer2(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
 #define ffetarget_xor_integer3(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
-#define ffetarget_xor_logical1(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
-#define ffetarget_xor_logical2(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
-#define ffetarget_xor_logical3(res,l,r) (*(res) = !!(l) != !!(r), FFEBAD)
+#define ffetarget_xor_integer4(res,l,r) (*(res) = (l) ^ (r), FFEBAD)
+#define ffetarget_xor_logical1(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_xor_logical2(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_xor_logical3(res,l,r) (*(res) = (l) != (r), FFEBAD)
+#define ffetarget_xor_logical4(res,l,r) (*(res) = (l) != (r), FFEBAD)
 
 /* End of #include file. */
 
