@@ -61,11 +61,13 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 /* Externals defined here. */
 
 int flag_traditional;		/* Shouldn't need this (C front end only)! */
+bool ffe_is_do_internal_checks_ = TRUE;
 bool ffe_is_90_ = FFETARGET_defaultIS_90;
 bool ffe_is_automatic_ = FFETARGET_defaultIS_AUTOMATIC;
 bool ffe_is_backslash_ = FFETARGET_defaultIS_BACKSLASH;
 bool ffe_is_underscoring_ = FFETARGET_defaultEXTERNAL_UNDERSCORED
   || FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED;
+bool ffe_is_second_underscore_ = FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED;
 bool ffe_is_dollar_ok_ = FFETARGET_defaultIS_DOLLAR_OK;
 bool ffe_is_f2c_ = FFETARGET_defaultIS_F2C;
 bool ffe_is_f2c_library_ = FFETARGET_defaultIS_F2C_LIBRARY;
@@ -83,6 +85,8 @@ bool ffe_is_ugly_init_ = FFETARGET_defaultIS_UGLY_INIT;
 bool ffe_is_version_ = FALSE;
 bool ffe_is_vxt_not_90_ = FFETARGET_defaultIS_VXT_NOT_90;
 bool ffe_is_warn_implicit_ = FALSE;
+bool ffe_is_warn_surprising_ = FALSE;
+bool ffe_is_zeros_ = FALSE;
 ffeCase ffe_case_intrin_ = FFETARGET_defaultCASE_INTRIN;
 ffeCase ffe_case_match_ = FFETARGET_defaultCASE_MATCH;
 ffeCase ffe_case_source_ = FFETARGET_defaultCASE_SOURCE;
@@ -155,6 +159,13 @@ ffe_decode_option (char *opt)
       else if (strcmp (&opt[2], "null-version") == 0)
 	;			/* Someday generate program to print version
 				   info.  */
+      else if (strcmp (&opt[2], "set-g77-defaults") == 0)
+	{
+	  ffe_is_do_internal_checks_ = 0;
+	  flag_move_all_movables = 1;
+	  flag_reduce_all_givs = 1;
+	  flag_rerun_loop_opt = 1;
+	}
       else if (strcmp (&opt[2], "ident") == 0)
 	ffe_set_is_ident (TRUE);
       else if (strcmp (&opt[2], "no-ident") == 0)
@@ -231,6 +242,10 @@ ffe_decode_option (char *opt)
 	ffe_set_is_underscoring (TRUE);
       else if (strcmp (&opt[2], "no-underscoring") == 0)
 	ffe_set_is_underscoring (FALSE);
+      else if (strcmp (&opt[2], "second-underscore") == 0)
+	ffe_set_is_second_underscore (TRUE);
+      else if (strcmp (&opt[2], "no-second-underscore") == 0)
+	ffe_set_is_second_underscore (FALSE);
       else if (strcmp (&opt[2], "intrin-case-initcap") == 0)
 	ffe_set_case_intrin (FFE_caseINITCAP);
       else if (strcmp (&opt[2], "intrin-case-upper") == 0)
@@ -351,6 +366,10 @@ ffe_decode_option (char *opt)
 	ffe_set_intrinsic_state_vxt (FFE_intrinsicstateDISABLED);
       else if (strcmp (&opt[2], "vxt-intrinsics-enable") == 0)
 	ffe_set_intrinsic_state_vxt (FFE_intrinsicstateENABLED);
+      else if (strcmp (&opt[2], "zeros") == 0)
+	ffe_set_is_zeros (TRUE);
+      else if (strcmp (&opt[2], "no-zeros") == 0)
+	ffe_set_is_zeros (FALSE);
       else if (strncmp (&opt[2], "fixed-line-length-",
 			strlen ("fixed-line-length-")) == 0)
 	{
@@ -385,9 +404,13 @@ ffe_decode_option (char *opt)
       else if (!strcmp (&opt[2], "no-import"))
 	; /* cpp handles this one.  */
       else if (!strcmp (&opt[2], "implicit"))
-	ffe_set_is_warn_implicit(TRUE);
+	ffe_set_is_warn_implicit (TRUE);
       else if (!strcmp (&opt[2], "no-implicit"))
-	ffe_set_is_warn_implicit(FALSE);
+	ffe_set_is_warn_implicit (FALSE);
+      else if (!strcmp (&opt[2], "surprising"))
+	ffe_set_is_warn_surprising (TRUE);
+      else if (!strcmp (&opt[2], "no-surprising"))
+	ffe_set_is_warn_surprising (FALSE);
       else if (!strcmp (&opt[2], "all"))
 	{
 	  /* We save the value of warn_uninitialized, since if they put
@@ -396,6 +419,7 @@ ffe_decode_option (char *opt)
 	  if (warn_uninitialized != 1)
 	    warn_uninitialized = 2;
 	  warn_unused = 1;
+	  ffe_set_is_warn_surprising (TRUE);
 	}
       else
 	return 0;
