@@ -78,10 +78,12 @@ typedef void *ffeUnionLongPtr;	/* unused type to cover union of long and
 
 /* Global objects accessed by users of this module. */
 
+extern bool ffe_is_do_internal_checks_;
 extern bool ffe_is_90_;
 extern bool ffe_is_automatic_;
 extern bool ffe_is_backslash_;
 extern bool ffe_is_underscoring_;
+extern bool ffe_is_second_underscore_;
 extern bool ffe_is_dollar_ok_;
 extern bool ffe_is_f2c_;
 extern bool ffe_is_f2c_library_;
@@ -98,6 +100,8 @@ extern bool ffe_is_ugly_init_;
 extern bool ffe_is_version_;
 extern bool ffe_is_vxt_not_90_;
 extern bool ffe_is_warn_implicit_;
+extern bool ffe_is_warn_surprising_;
+extern bool ffe_is_zeros_;
 extern ffeCase ffe_case_intrin_;
 extern ffeCase ffe_case_match_;
 extern ffeCase ffe_case_source_;
@@ -153,6 +157,7 @@ void ffe_terminate_4 (void);
 #define ffe_is_90() ffe_is_90_
 #define ffe_is_automatic() ffe_is_automatic_
 #define ffe_is_backslash() ffe_is_backslash_
+#define ffe_is_do_internal_checks() ffe_is_do_internal_checks_
 #define ffe_is_dollar_ok() ffe_is_dollar_ok_
 #define ffe_is_f2c() ffe_is_f2c_
 #define ffe_is_f2c_library() ffe_is_f2c_library_
@@ -164,6 +169,7 @@ void ffe_terminate_4 (void);
 #define ffe_is_pedantic() ffe_is_pedantic_
 #define ffe_is_pedantic_not_90() (ffe_is_pedantic_ && !ffe_is_90_)
 #define ffe_is_saveall() ffe_is_saveall_
+#define ffe_is_second_underscore() ffe_is_second_underscore_
 #define ffe_is_ugly() ffe_is_ugly_
 #define ffe_is_ugly_args() ffe_is_ugly_args_
 #define ffe_is_ugly_init() ffe_is_ugly_init_
@@ -171,6 +177,8 @@ void ffe_terminate_4 (void);
 #define ffe_is_version() ffe_is_version_
 #define ffe_is_vxt_not_90() ffe_is_vxt_not_90_
 #define ffe_is_warn_implicit() ffe_is_warn_implicit_
+#define ffe_is_warn_surprising() ffe_is_warn_surprising_
+#define ffe_is_zeros() ffe_is_zeros_
 #define ffe_fixed_line_length() ffe_fixed_line_length_
 #define ffe_pool_file() (ffe_file_pool_)
 #define ffe_pool_any_unit() (ffe_any_unit_pool_)
@@ -188,6 +196,7 @@ void ffe_terminate_4 (void);
 #define ffe_set_is_90(f) (ffe_is_90_ = (f))
 #define ffe_set_is_automatic(f) (ffe_is_automatic_ = (f))
 #define ffe_set_is_backslash(f) (ffe_is_backslash_ = (f))
+#define ffe_set_is_do_internal_checks(f) (ffe_set_is_do_internal_checks_ = (f))
 #define ffe_set_is_dollar_ok(f) (ffe_is_dollar_ok_ = (f))
 #define ffe_set_is_f2c(f) (ffe_is_f2c_ = (f))
 #define ffe_set_is_f2c_library(f) (ffe_is_f2c_library_ = (f))
@@ -198,6 +207,7 @@ void ffe_terminate_4 (void);
 #define ffe_set_is_mainprog(f) (ffe_is_mainprog_ = (f))
 #define ffe_set_is_pedantic(f) (ffe_is_pedantic_ = (f))
 #define ffe_set_is_saveall(f) (ffe_is_saveall_ = (f))
+#define ffe_set_is_second_underscore(f) (ffe_is_second_underscore_ = (f))
 #define ffe_set_is_ugly(f) (ffe_is_ugly_ = (f))
 #define ffe_set_is_ugly_args(f) (ffe_is_ugly_args_ = (f))
 #define ffe_set_is_ugly_init(f) (ffe_is_ugly_init_ = (f))
@@ -205,6 +215,8 @@ void ffe_terminate_4 (void);
 #define ffe_set_is_version(f) (ffe_is_version_ = (f))
 #define ffe_set_is_vxt_not_90(f) (ffe_is_vxt_not_90_ = (f))
 #define ffe_set_is_warn_implicit(f) (ffe_is_warn_implicit_ = (f))
+#define ffe_set_is_warn_surprising(f) (ffe_is_warn_surprising_ = (f))
+#define ffe_set_is_zeros(f) (ffe_is_zeros_ = (f))
 #define ffe_set_fixed_line_length(l) (ffe_fixed_line_length_ = (l))
 #define ffe_state_max(s1,s2) ((s1) > (s2) ? (s1) : (s2))
 
