@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Owning Modules:
       zzz.c
@@ -24,35 +25,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    Modifications:
 */
 
-/* Allow multiple inclusion to work. */
-
 #ifndef _H_f_zzz
 #define _H_f_zzz
 
-/* Simple definitions and enumerations. */
-
-
-/* Typedefs. */
-
-
-/* Include files needed by this one. */
-
-
-/* Structure definitions. */
-
-
-/* Global objects accessed by users of this module. */
-
 extern char *ffezzz_version_string;
 extern char *ffezzz_date;
 extern char *ffezzz_time;
 
-/* Declare functions with prototypes. */
-
-
-/* Define macros. */
-
-
-/* End of #include file. */
-
 #endif
